{# Status badge fragment - expects 'value' variable in scope #}
{% if value %}
<span class="badge badge-sm {{ value | badge_class }}">{{ value | replace("_", " ") | title }}</span>
{% else %}
<span class="text-base-content/30">—</span>
{% endif %}
