"""
DNR-Back Core

Framework-agnostic backend abstraction layer.

This is the "kernel" - pure Python abstractions that are independent of any framework.
Pydantic, FastAPI, etc. are adapters/builders that consume BackendSpec.
"""

# Placeholder for DazzleModel and core abstractions
