/* =============================================================================
   DAZZLE Site Sections

   Styles for SiteSpec page sections (hero, features, pricing, FAQ, etc.)
   Uses CSS custom properties from the theme system.
   ============================================================================= */

/* =============================================================================
   Hero Section
   ============================================================================= */

.dz-section-hero {
  background: linear-gradient(135deg, var(--dz-hero-bg-from) 0%, var(--dz-hero-bg-to) 100%);
  color: white;
  padding: var(--dz-spacing-hero-y, 6rem) 1.5rem;
  text-align: center;
  position: relative;
  overflow: hidden;
}

.dz-section-hero::before {
  content: '';
  position: absolute;
  top: 0;
  left: 0;
  right: 0;
  bottom: 0;
  background: radial-gradient(circle at 30% 70%, oklch(1 0 0 / 0.1) 0%, transparent 50%);
  pointer-events: none;
}

/* Hero with Media Image — flex on the section to create two columns */
.dz-hero-with-media {
  display: flex;
  align-items: center;
  gap: 3rem;
}

.dz-hero-with-media .dz-section-content {
  flex: 1;
  text-align: left;
}

.dz-hero-with-media .dz-cta-group {
  justify-content: flex-start;
}

.dz-hero-media {
  flex: 1;
  max-width: 50%;
}

.dz-hero-image {
  width: 100%;
  height: auto;
  border-radius: var(--dz-radius-lg, 1rem);
  box-shadow: var(--dz-shadow-xl, 0 20px 50px -12px oklch(0 0 0 / 0.3));
}

@media (max-width: 768px) {
  .dz-hero-with-media {
    flex-direction: column;
    text-align: center;
  }

  .dz-hero-with-media .dz-section-content {
    text-align: center;
  }

  .dz-hero-with-media .dz-cta-group {
    justify-content: center;
  }

  .dz-hero-media {
    max-width: 100%;
    order: -1;
  }
}

/* =============================================================================
   Features Section
   ============================================================================= */

.dz-section-features {
  padding: var(--dz-spacing-section-y, 5rem) 1.5rem;
  background: var(--p, oklch(0.99 0 0));
}

/* =============================================================================
   Pricing Section
   ============================================================================= */

.dz-section-pricing {
  padding: var(--dz-spacing-section-y, 5rem) 1.5rem;
  background: var(--dz-section-alt-bg, oklch(0.97 0.01 260));
}

/* =============================================================================
   FAQ Section
   ============================================================================= */

.dz-section-faq {
  padding: var(--dz-spacing-section-y, 5rem) 1.5rem;
}

/* =============================================================================
   Testimonials Section
   ============================================================================= */

.dz-section-testimonials {
  padding: var(--dz-spacing-section-y, 5rem) 1.5rem;
  background: var(--dz-section-alt-bg, oklch(0.97 0.01 260));
}

/* =============================================================================
   Stats Section
   ============================================================================= */

.dz-section-stats {
  padding: var(--dz-spacing-section-y, 5rem) 1.5rem;
  background: linear-gradient(135deg, var(--dz-hero-bg-from) 0%, var(--dz-hero-bg-to) 100%);
  color: white;
}

/* =============================================================================
   CTA Section
   ============================================================================= */

.dz-section-cta {
  padding: var(--dz-spacing-section-y, 5rem) 1.5rem;
  background: var(--dz-section-alt-bg, oklch(0.97 0.01 260));
  text-align: center;
}

.dz-section-cta h2 {
  font-size: var(--dz-font-size-cta-headline, 3rem);
  font-weight: 800;
  letter-spacing: -0.02em;
}

/* =============================================================================
   Steps Section
   ============================================================================= */

.dz-section-steps {
  padding: var(--dz-spacing-section-y, 5rem) 1.5rem;
}

/* =============================================================================
   Logo Cloud Section
   ============================================================================= */

.dz-section-logo-cloud {
  padding: var(--dz-spacing-section-y, 5rem) 1.5rem;
  background: var(--dz-section-alt-bg, oklch(0.97 0.01 260));
  text-align: center;
}

/* =============================================================================
   Comparison Section
   ============================================================================= */

.dz-section-comparison {
  padding: var(--dz-spacing-section-y, 5rem) 1.5rem;
}

.dz-comparison-wrapper {
  overflow-x: auto;
  -webkit-overflow-scrolling: touch;
}

.dz-comparison-table {
  width: 100%;
  border-collapse: collapse;
  text-align: center;
}

.dz-comparison-table th,
.dz-comparison-table td {
  padding: 0.75rem 1rem;
  border-bottom: 1px solid var(--dz-border-subtle);
}

.dz-comparison-table thead th {
  font-weight: 600;
  font-size: 1rem;
  padding-bottom: 1rem;
}

.dz-comparison-feature {
  text-align: left;
  font-weight: 500;
}

.dz-comparison-highlighted {
  background: var(--dz-feature-icon-bg, oklch(0.92 0.03 260));
}

/* =============================================================================
   Value Highlight Section
   ============================================================================= */

.dz-section-value-highlight {
  padding: var(--dz-spacing-hero-y, 6rem) 1.5rem;
  text-align: center;
  background: var(--dz-section-alt-bg, oklch(0.97 0.01 260));
}

.dz-value-headline {
  font-size: var(--dz-font-size-hero-headline, 3.5rem);
  font-weight: var(--dz-font-weight-hero-headline, 800);
  line-height: var(--dz-line-height-hero-headline, 1.1);
  letter-spacing: var(--dz-letter-spacing-hero-headline, -0.02em);
}

.dz-value-body {
  max-width: 40rem;
  margin: 0 auto 2rem;
  font-size: 1.125rem;
  line-height: 1.6;
  color: var(--dz-text-body);
}

/* =============================================================================
   Split Content Section
   ============================================================================= */

.dz-section-split-content {
  padding: var(--dz-spacing-section-y, 5rem) 1.5rem;
}

.dz-split-grid {
  display: grid;
  grid-template-columns: 1fr 1fr;
  gap: 3rem;
  align-items: center;
}

.dz-split--reversed .dz-split-grid {
  direction: rtl;
}

.dz-split--reversed .dz-split-grid > * {
  direction: ltr;
}

.dz-split-text h2 {
  text-align: left;
}

.dz-split-text p {
  color: var(--dz-text-body);
}

.dz-cta-group--left {
  justify-content: flex-start;
}

.dz-split-media img {
  width: 100%;
  height: auto;
  border-radius: var(--dz-radius-lg, 1rem);
  box-shadow: var(--dz-shadow-card, 0 1px 3px oklch(0 0 0 / 0.1));
}

@media (max-width: 768px) {
  .dz-split-grid {
    grid-template-columns: 1fr;
  }

  .dz-split--reversed .dz-split-grid {
    direction: ltr;
  }

  .dz-split-text h2 {
    text-align: center;
  }

  .dz-cta-group--left {
    justify-content: center;
  }
}

/* =============================================================================
   Card Grid Section
   ============================================================================= */

.dz-section-card-grid {
  padding: var(--dz-spacing-section-y, 5rem) 1.5rem;
  background: var(--dz-section-alt-bg, oklch(0.97 0.01 260));
}

.dz-card-grid {
  display: grid;
  grid-template-columns: repeat(auto-fit, minmax(280px, 1fr));
  gap: 1.5rem;
}

.dz-card-item {
  background: var(--dz-surface-card);
  padding: var(--dz-spacing-card-padding, 1.5rem);
  border-radius: var(--dz-radius-card, 0.75rem);
  box-shadow: var(--dz-shadow-card, 0 1px 3px oklch(0 0 0 / 0.1));
  display: flex;
  flex-direction: column;
  transition: transform var(--dz-transition-fast, 150ms),
              box-shadow var(--dz-transition-fast, 150ms);
}

.dz-card-item:hover {
  transform: translateY(-4px);
  box-shadow: var(--dz-shadow-lg, 0 10px 15px -3px oklch(0 0 0 / 0.1));
}

.dz-card-item h3 {
  font-size: 1.125rem;
  font-weight: 600;
  margin: 0.5rem 0;
}

.dz-card-item p {
  color: var(--dz-text-body);
  margin: 0 0 1rem;
  line-height: var(--dz-line-height-body, 1.6);
  flex: 1;
}

.dz-card-icon {
  width: 3rem;
  height: 3rem;
  background: var(--dz-feature-icon-bg, oklch(0.92 0.03 260));
  border-radius: 0.5rem;
  display: flex;
  align-items: center;
  justify-content: center;
  margin-bottom: 1rem;
  color: var(--dz-hero-bg-from);
}

.dz-card-icon svg {
  width: 1.5rem;
  height: 1.5rem;
}

/* =============================================================================
   Trust Bar Section
   ============================================================================= */

.dz-section-trust-bar {
  padding: 2rem 1.5rem;
  background: var(--dz-section-alt-bg, oklch(0.97 0.01 260));
  border-top: 1px solid var(--dz-border-subtle);
  border-bottom: 1px solid var(--dz-border-subtle);
}

.dz-trust-strip {
  display: flex;
  flex-wrap: wrap;
  justify-content: center;
  gap: 2rem;
  align-items: center;
}

.dz-trust-item {
  display: flex;
  align-items: center;
  gap: 0.5rem;
  font-size: 0.875rem;
  font-weight: 500;
  color: var(--dz-text-body);
}

.dz-trust-item svg {
  width: 1.25rem;
  height: 1.25rem;
  color: var(--dz-hero-bg-from);
}

/* =============================================================================
   Dark Mode Adjustments
   ============================================================================= */

/* Site Header — dark mode handled by token overrides in design-system.css */

/* Theme Toggle Button */
.dz-theme-toggle {
  background: transparent;
  border: none;
  cursor: pointer;
  padding: 0.5rem;
  border-radius: 0.375rem;
  color: var(--dz-text-body);
  transition: color var(--dz-transition-fast, 150ms), background var(--dz-transition-fast, 150ms);
  display: flex;
  align-items: center;
  justify-content: center;
}

.dz-theme-toggle:hover {
  color: var(--dz-text-foreground);
  background: var(--dz-border-subtle);
}

/* Theme toggle dark mode handled by token overrides in design-system.css */

.dz-theme-toggle__icon {
  width: 1.25rem;
  height: 1.25rem;
}

/* Show sun icon by default (for dark mode), hide moon */
.dz-theme-toggle__sun { display: none; }
.dz-theme-toggle__moon { display: block; }

/* In dark mode, show sun, hide moon */
[data-theme="dark"] .dz-theme-toggle__sun { display: block; }
[data-theme="dark"] .dz-theme-toggle__moon { display: none; }

/* Section backgrounds */
[data-theme="dark"] .dz-section-features,
[data-theme="dark"] .dz-section-faq,
[data-theme="dark"] .dz-section-steps {
  background: oklch(0.12 0.01 260);
}

[data-theme="dark"] .dz-section-pricing,
[data-theme="dark"] .dz-section-testimonials,
[data-theme="dark"] .dz-section-logo-cloud,
[data-theme="dark"] .dz-section-cta,
[data-theme="dark"] .dz-section-value-highlight,
[data-theme="dark"] .dz-section-card-grid,
[data-theme="dark"] .dz-section-trust-bar {
  background: oklch(0.08 0.01 260);
}

[data-theme="dark"] .dz-section-comparison,
[data-theme="dark"] .dz-section-split-content {
  background: oklch(0.12 0.01 260);
}

/* Cards and items */
[data-theme="dark"] .dz-feature-item,
[data-theme="dark"] .dz-testimonial-item,
[data-theme="dark"] .dz-card-item,
[data-theme="dark"] .dz-pricing-tier:not(.dz-pricing-highlighted) {
  background: oklch(0.18 0.01 260);
}

/* Comparison table */
[data-theme="dark"] .dz-comparison-table th,
[data-theme="dark"] .dz-comparison-table td {
  border-color: oklch(1 0 0 / 0.1);
}

[data-theme="dark"] .dz-comparison-highlighted {
  background: oklch(0.18 0.03 260);
}

/* Headings */
[data-theme="dark"] .dz-section h2,
[data-theme="dark"] .dz-feature-item h3,
[data-theme="dark"] .dz-step-item h3,
[data-theme="dark"] .dz-faq-item summary,
[data-theme="dark"] .dz-pricing-tier h3 {
  color: oklch(0.95 0 0);
}

/* Body text */
[data-theme="dark"] .dz-feature-item p,
[data-theme="dark"] .dz-step-item p,
[data-theme="dark"] .dz-testimonial-item blockquote,
[data-theme="dark"] .dz-faq-item p {
  color: oklch(0.65 0 0);
}

/* FAQ borders */
[data-theme="dark"] .dz-faq-item {
  border-color: oklch(1 0 0 / 0.1);
}

/* Auth pages - works with both DaisyUI card and legacy dz-auth-card */
[data-theme="dark"] .dz-auth-card {
  background: oklch(0.18 0.01 260);
}

[data-theme="dark"] .dz-auth-card h1,
[data-theme="dark"] .dz-auth-card .card-title,
[data-theme="dark"] .dz-auth-field label,
[data-theme="dark"] .dz-auth-card .label-text {
  color: oklch(0.9 0 0);
}

[data-theme="dark"] .dz-auth-field input,
[data-theme="dark"] .dz-auth-card .input {
  background: oklch(0.12 0.01 260);
  border-color: oklch(1 0 0 / 0.15);
  color: oklch(0.95 0 0);
}

[data-theme="dark"] .dz-auth-switch {
  color: oklch(0.6 0 0);
}

/* Prose/markdown content */
[data-theme="dark"] .dz-prose {
  color: oklch(0.85 0 0);
}

[data-theme="dark"] .dz-prose h1,
[data-theme="dark"] .dz-prose h2,
[data-theme="dark"] .dz-prose h3 {
  color: oklch(0.95 0 0);
}

/* Logo cloud */
[data-theme="dark"] .dz-logo-item img {
  filter: grayscale(100%) invert(1);
  opacity: 0.5;
}

[data-theme="dark"] .dz-logo-item:hover img {
  filter: grayscale(0%) invert(1);
  opacity: 0.8;
}

/* Buttons - works with both DaisyUI btn and legacy dz-btn classes */
[data-theme="dark"] .dz-btn-secondary,
[data-theme="dark"] .btn-outline {
  color: oklch(0.85 0 0);
  border-color: oklch(0.85 0 0);
}

[data-theme="dark"] .dz-btn-secondary:hover,
[data-theme="dark"] .btn-outline:hover {
  background: oklch(1 0 0 / 0.1);
}

/* =============================================================================
   Responsive Adjustments
   ============================================================================= */

@media (max-width: 768px) {
  .dz-section h1 {
    font-size: 2.5rem;
  }

  .dz-section h2 {
    font-size: 1.75rem;
  }

  .dz-section-cta h2 {
    font-size: 2.25rem;
  }

  .dz-cta-group {
    flex-direction: column;
    align-items: center;
  }

  .dz-pricing-highlighted {
    transform: none;
  }

  .dz-stat-value {
    font-size: 2.25rem;
  }
}

/* =============================================================================
   Site Layout - Header, Footer, Nav
   ============================================================================= */

.dz-site {
  display: flex;
  flex-direction: column;
  min-height: 100vh;
}

.dz-site-header {
  position: sticky;
  top: 0;
  z-index: 100;
  background: var(--dz-header-bg);
  backdrop-filter: blur(10px);
  border-bottom: 1px solid var(--dz-border-subtle);
}

.dz-site-nav {
  max-width: 72rem;
  margin: 0 auto;
  padding: 1rem 1.5rem;
  display: flex;
  align-items: center;
  justify-content: space-between;
  gap: 2rem;
}

.dz-site-logo {
  font-size: 1.25rem;
  font-weight: 700;
  color: var(--dz-logo-color);
  text-decoration: none;
}

.dz-nav-items {
  display: flex;
  align-items: center;
  gap: 1.5rem;
}

.dz-nav-link {
  font-size: 0.875rem;
  font-weight: 500;
  color: var(--dz-text-body);
  text-decoration: none;
  transition: color var(--dz-transition-fast, 150ms);
}

.dz-nav-link:hover {
  color: var(--dz-text-foreground);
}

.dz-nav-cta {
  font-size: 0.875rem;
  font-weight: 600;
  color: white;
  background: var(--dz-hero-bg-from, oklch(0.50 0.18 270));
  padding: 0.5rem 1rem;
  border-radius: var(--dz-radius-button, 0.375rem);
  text-decoration: none;
  transition: background var(--dz-transition-fast, 150ms);
}

.dz-nav-cta:hover {
  background: var(--dz-hero-bg-to, oklch(0.45 0.18 270));
}

.dz-site-footer {
  margin-top: auto;
  background: var(--dz-footer-bg);
  color: var(--dz-footer-text);
  padding: 3rem 1.5rem 1.5rem;
}

.dz-footer-content {
  max-width: 72rem;
  margin: 0 auto;
  display: grid;
  grid-template-columns: repeat(auto-fit, minmax(150px, 1fr));
  gap: 2rem;
  margin-bottom: 2rem;
}

.dz-footer-col h4 {
  font-size: 0.875rem;
  font-weight: 600;
  color: white;
  margin: 0 0 1rem;
}

.dz-footer-col ul {
  list-style: none;
  padding: 0;
  margin: 0;
}

.dz-footer-col li {
  margin-bottom: 0.5rem;
}

.dz-footer-col a {
  font-size: 0.875rem;
  color: var(--dz-footer-link);
  text-decoration: none;
  transition: color var(--dz-transition-fast, 150ms);
}

.dz-footer-col a:hover {
  color: white;
}

.dz-footer-bottom {
  max-width: 72rem;
  margin: 0 auto;
  padding-top: 1.5rem;
  border-top: 1px solid var(--dz-footer-border);
  text-align: center;
}

.dz-footer-bottom p {
  font-size: 0.75rem;
  margin: 0;
}

/* =============================================================================
   Auth Pages (Login/Signup)
   ============================================================================= */

.dz-auth-page {
  background: linear-gradient(135deg, var(--dz-hero-bg-from, oklch(0.50 0.18 270)) 0%, var(--dz-hero-bg-to, oklch(0.45 0.18 270)) 100%);
  display: flex;
  align-items: center;
  justify-content: center;
  min-height: 100vh;
  padding: 2rem 1rem;
}

.dz-auth-container {
  width: 100%;
  max-width: 24rem;
}

.dz-auth-card {
  background: var(--dz-surface-card);
  border-radius: var(--dz-radius-lg, 0.75rem);
  padding: 2rem;
  box-shadow: var(--dz-shadow-xl, 0 20px 50px -12px oklch(0 0 0 / 0.25));
  text-align: center;
}

.dz-auth-logo {
  font-size: 1.5rem;
  font-weight: 700;
  color: var(--dz-hero-bg-from, oklch(0.50 0.18 270));
  text-decoration: none;
  display: inline-block;
  margin-bottom: 1.5rem;
}

.dz-auth-card h1 {
  font-size: 1.5rem;
  font-weight: 600;
  color: var(--dz-text-foreground);
  margin: 0 0 1.5rem;
}

.dz-auth-error {
  background: var(--dz-error-bg);
  color: var(--dz-error-text);
  padding: 0.75rem 1rem;
  border-radius: var(--dz-radius-sm, 0.375rem);
  font-size: 0.875rem;
  margin-bottom: 1rem;
  text-align: left;
}

.dz-auth-error.hidden {
  display: none;
}

#dz-auth-form {
  text-align: left;
}

.dz-auth-field {
  margin-bottom: 1rem;
}

.dz-auth-field label {
  display: block;
  font-size: 0.875rem;
  font-weight: 500;
  color: var(--dz-text-emphasis);
  margin-bottom: 0.375rem;
}

/* NOTE: Auth form fields now use DaisyUI input input-bordered classes */
/* These styles kept for backwards compatibility */
.dz-auth-field input {
  width: 100%;
  padding: 0.625rem 0.75rem;
  font-size: 1rem;
  border: 1px solid var(--dz-border-input);
  border-radius: var(--dz-radius-sm, 0.375rem);
  transition: border-color var(--dz-transition-fast, 150ms), box-shadow var(--dz-transition-fast, 150ms);
  box-sizing: border-box;
}

.dz-auth-field input:focus {
  outline: none;
  border-color: var(--dz-hero-bg-from, oklch(0.50 0.18 270));
  box-shadow: var(--dz-ring-glow);
}

/* NOTE: .dz-btn-full replaced by Tailwind w-full class */

/* Auth page primary button - ensure visibility with DaisyUI + Tailwind Browser */
.dz-auth-card .btn-primary,
.dz-auth-card .btn.btn-primary {
  background: var(--dz-hero-bg-from, oklch(0.50 0.18 270));
  color: white;
  border: none;
}

.dz-auth-card .btn-primary:hover,
.dz-auth-card .btn.btn-primary:hover {
  background: var(--dz-hero-bg-to, oklch(0.45 0.18 270));
}

.dz-auth-switch {
  margin-top: 1.5rem;
  font-size: 0.875rem;
  color: var(--dz-text-muted);
}

.dz-auth-switch a {
  color: var(--dz-hero-bg-from, oklch(0.50 0.18 270));
  text-decoration: none;
  font-weight: 500;
}

.dz-auth-switch a:hover {
  text-decoration: underline;
}

/* =============================================================================
   Button Styles for Site Pages

   NOTE: Most button styling is now handled by DaisyUI btn classes (btn, btn-primary, etc.)
   These are kept for backwards compatibility and site-specific overrides.
   ============================================================================= */

/* Hero section CTA buttons need special styling to stand out on gradient background */
.dz-section-hero .btn-primary {
  background: white;
  color: var(--dz-hero-bg-from, oklch(0.50 0.18 270));
  border: none;
}

.dz-section-hero .btn-primary:hover {
  background: var(--dz-hero-btn-hover);
  transform: translateY(-2px);
}

.dz-section-hero .btn-secondary,
.dz-section-hero .btn-outline {
  background: transparent;
  color: white;
  border-color: var(--dz-hero-border);
}

.dz-section-hero .btn-secondary:hover,
.dz-section-hero .btn-outline:hover {
  background: var(--dz-hero-overlay);
  border-color: var(--dz-hero-border-hover);
}

/* =============================================================================
   Site Page Sections (JS-rendered)

   These rules target the HTML structure generated by get_site_js().
   They use design-system tokens so themes propagate correctly.
   ============================================================================= */

.dz-section {
  width: 100%;
}

.dz-section-content {
  max-width: 72rem;
  margin: 0 auto;
  padding: 0 1.5rem;
}

/* --- Section header (renderSectionHeader helper) --- */
.dz-section-header {
  text-align: center;
  max-width: 40rem;
  margin: 0 auto 3rem;
}

/* --- Generic section typography --- */
.dz-section h1 {
  font-size: var(--dz-font-size-hero-headline, 3.5rem);
  font-weight: var(--dz-font-weight-hero-headline, 800);
  line-height: var(--dz-line-height-hero-headline, 1.1);
  letter-spacing: var(--dz-letter-spacing-hero-headline, -0.02em);
  margin: 0 0 1.5rem;
}

.dz-section h2 {
  font-size: var(--dz-font-size-section-headline, 2.25rem);
  font-weight: var(--dz-font-weight-section-headline, 700);
  line-height: var(--dz-line-height-section-headline, 1.2);
  letter-spacing: var(--dz-letter-spacing-section-headline, -0.01em);
  margin: 0 0 2rem;
  text-align: center;
}

.dz-section h3 {
  font-size: 1.125rem;
  font-weight: 600;
  margin: 0.5rem 0;
}

.dz-section p {
  font-size: var(--dz-font-size-body, 1rem);
  line-height: var(--dz-line-height-body, 1.6);
}

/* --- 404 page --- */
.dz-404-section {
  text-align: center;
  padding: var(--dz-spacing-hero-y, 6rem) 1.5rem;
}

.dz-404-headline {
  font-size: var(--dz-font-size-hero-headline, 3.5rem);
  font-weight: var(--dz-font-weight-hero-headline, 800);
  line-height: var(--dz-line-height-hero-headline, 1.1);
  margin-bottom: 1rem;
}

.dz-404-cta {
  margin-top: 2rem;
}

/* --- Pricing tier description --- */
.dz-pricing-description {
  color: var(--dz-text-muted);
  margin: 0 0 1.5rem;
  font-size: var(--dz-font-size-body, 1rem);
  line-height: var(--dz-line-height-body, 1.6);
}

/* --- Markdown page section --- */
.dz-section-markdown {
  padding: var(--dz-spacing-section-y, 5rem) 1.5rem;
}

.dz-subhead {
  font-size: 1.25rem;
  line-height: 1.6;
  opacity: 0.9;
  margin: 0 0 2rem;
}

.dz-cta-group {
  display: flex;
  gap: 1rem;
  justify-content: center;
  flex-wrap: wrap;
}

.dz-features-grid,
.dz-testimonials-grid {
  display: grid;
  grid-template-columns: repeat(auto-fit, minmax(280px, 1fr));
  gap: 1.5rem;
}

.dz-pricing-grid {
  display: grid;
  grid-template-columns: repeat(auto-fit, minmax(220px, 1fr));
  gap: 1.5rem;
}

.dz-feature-item,
.dz-testimonial-item {
  background: var(--dz-surface-card);
  padding: var(--dz-spacing-card-padding, 1.5rem);
  border-radius: var(--dz-radius-card, 0.75rem);
  box-shadow: var(--dz-shadow-card, 0 1px 3px oklch(0 0 0 / 0.1));
  transition: transform var(--dz-transition-fast, 150ms), box-shadow var(--dz-transition-fast, 150ms);
}

.dz-feature-item:hover {
  transform: translateY(-4px);
  box-shadow: var(--dz-shadow-lg, 0 10px 15px -3px oklch(0 0 0 / 0.1));
}

.dz-feature-item h3 {
  font-size: 1.125rem;
  font-weight: 600;
  margin: 0.5rem 0;
}

.dz-feature-item p {
  color: var(--dz-text-body);
  margin: 0;
  line-height: var(--dz-line-height-body, 1.6);
}

.dz-feature-icon {
  width: 3rem;
  height: 3rem;
  background: var(--dz-feature-icon-bg, oklch(0.92 0.03 260));
  border-radius: 0.5rem;
  display: flex;
  align-items: center;
  justify-content: center;
  margin-bottom: 1rem;
  color: var(--dz-hero-bg-from);
}

.dz-feature-icon svg {
  width: 1.5rem;
  height: 1.5rem;
}

.dz-faq-list {
  max-width: 48rem;
  margin: 0 auto;
}

.dz-faq-item {
  border-bottom: 1px solid var(--dz-border-default);
  padding: 1rem 0;
}

.dz-faq-item summary {
  font-weight: 600;
  cursor: pointer;
  list-style: none;
}

.dz-faq-item summary::-webkit-details-marker {
  display: none;
}

.dz-faq-item p {
  margin: 0.75rem 0 0;
  color: var(--dz-text-body);
  line-height: var(--dz-line-height-body, 1.6);
}

.dz-stats-grid {
  display: grid;
  grid-template-columns: repeat(auto-fit, minmax(120px, 1fr));
  gap: 2rem;
  text-align: center;
}

.dz-stat-item {
  padding: 1rem;
}

.dz-stat-value {
  font-size: 2.5rem;
  font-weight: 700;
  display: block;
}

.dz-stat-label {
  font-size: 0.875rem;
  opacity: 0.8;
}

.dz-steps-list {
  max-width: 48rem;
  margin: 0 auto;
  display: flex;
  flex-direction: column;
  gap: 2rem;
}

.dz-step-item {
  display: flex;
  gap: 1.5rem;
  align-items: flex-start;
}

.dz-step-number {
  width: 3rem;
  height: 3rem;
  background: var(--dz-hero-bg-from);
  color: white;
  border-radius: 50%;
  display: flex;
  align-items: center;
  justify-content: center;
  font-weight: 700;
  font-size: 1.25rem;
  flex-shrink: 0;
}

.dz-step-item h3 {
  font-size: 1.25rem;
  font-weight: 600;
  margin: 0 0 0.5rem;
}

.dz-step-item p {
  color: var(--dz-text-body);
  margin: 0;
  line-height: var(--dz-line-height-body, 1.6);
}

.dz-logos-grid {
  display: flex;
  flex-wrap: wrap;
  justify-content: center;
  gap: 3rem;
  align-items: center;
}

.dz-logo-item img {
  height: 2rem;
  opacity: 0.6;
  filter: grayscale(100%);
  transition: all var(--duration-fast, 150ms);
}

.dz-logo-item:hover img {
  opacity: 1;
  filter: grayscale(0%);
}

.dz-pricing-tier {
  background: var(--dz-surface-card);
  padding: 2rem;
  border-radius: var(--dz-radius-card, 0.75rem);
  box-shadow: var(--dz-shadow-card, 0 1px 3px oklch(0 0 0 / 0.1));
  display: flex;
  flex-direction: column;
  transition: transform var(--dz-transition-fast, 150ms);
}

.dz-pricing-tier:hover:not(.dz-pricing-highlighted) {
  transform: translateY(-4px);
}

.dz-pricing-tier h3 {
  font-size: 1.25rem;
  font-weight: 600;
  margin: 0 0 0.5rem;
}

.dz-pricing-price {
  margin: 0 0 1.5rem;
}

.dz-price {
  font-size: 2.5rem;
  font-weight: 700;
}

.dz-period {
  font-size: 0.875rem;
  color: var(--dz-text-muted);
}

.dz-pricing-features {
  list-style: none;
  padding: 0;
  margin: 0 0 1.5rem;
  flex: 1;
}

.dz-pricing-features li {
  padding: 0.5rem 0;
  border-bottom: 1px solid var(--dz-border-subtle);
}

.dz-pricing-highlighted {
  background: var(--dz-pricing-highlight-bg, var(--dz-hero-bg-from));
  color: white;
  transform: scale(1.05);
  box-shadow: var(--dz-shadow-lg);
}

.dz-pricing-highlighted .dz-period,
.dz-pricing-highlighted li {
  color: var(--dz-hero-text-dimmed);
  border-color: var(--dz-hero-border-dim);
}

.dz-testimonial-item:hover {
  transform: translateY(-4px);
  box-shadow: var(--dz-shadow-lg, 0 10px 15px -3px oklch(0 0 0 / 0.1));
}

.dz-testimonial-item blockquote {
  font-style: italic;
  color: var(--dz-text-emphasis);
  margin: 0 0 1rem;
  line-height: var(--dz-line-height-body, 1.6);
}

.dz-testimonial-author {
  display: flex;
  align-items: center;
  gap: 0.75rem;
}

.dz-avatar {
  width: 2.5rem;
  height: 2.5rem;
  border-radius: 50%;
}

.dz-testimonial-author strong {
  display: block;
  font-size: 0.875rem;
}

.dz-testimonial-author span {
  display: block;
  font-size: 0.75rem;
  color: var(--dz-text-muted);
}

.dz-prose {
  max-width: 48rem;
  margin: 0 auto;
  padding: 3rem 1.5rem;
}

.dz-prose h1, .dz-prose h2, .dz-prose h3 {
  text-align: left;
}

/* --- Task surface container (human task workflow pages) --- */
.dz-task-surface-container {
  max-width: 75rem;
  margin: 0 auto;
  padding: var(--dz-spacing-card-padding, 1.5rem);
}

.dz-loading {
  text-align: center;
  padding: 4rem;
  color: var(--dz-text-muted);
}

.dz-error {
  text-align: center;
  padding: 2rem;
  color: var(--dz-error-text);
}
