"""
DAZZLE UI Generation Module.

This module provides UI generation capabilities including:
- Semantic layout engine (v0.3.0)
- Stack-specific renderers
"""

from dazzle.ui.layout_engine import (
    ArchetypeDefinition,
    build_layout_plan,
    select_archetype,
)

__all__ = [
    "ArchetypeDefinition",
    "build_layout_plan",
    "select_archetype",
]
